@echo off & setlocal enableextensions enabledelayedexpansion
::
:: ./BatchSkeleton.bat
::
:: A Windows cmd skeleton Batch script.
::
:: Author: Martin J. Moene
::
:: $Id: $
::

:: Issue "BatchSkeleton --help" for more information on usage.
::
:: Note sometimes the & command chaining character is placed directly
:: adjacent to its preceding set command. This prevents a trailing
:: space to be appended to the variable.
::
:: For examples of batch programming, see:
::    http://www.netikka.net/tsneti/info/tscmd.php
::

::
:: A. Configuration
::
set          version=0.5
set      versionDate=$Date: 2011-03-02 11:08:59 +0100 (Wed, 02 Mar 2011) $
set      versionDate=%versionDate:~34,-3%& rem get the part inside the parentheses

set         progname=%0
set          logname=%0.log
set          logpath=%~f0.log
set        cachepath=%~f0.cache

::
:: B. Process Options, see :Usage
::
:: See Config section above for default values.
::
set /a optCount=0
:OptLoop
set argv=%1
set dash=%argv:~0,1%
if     [%argv%]==[]  goto :EndOptLoop
if not [%dash%]==[-] goto :EndOptLoop
   set /a optCount+=1

   if %argv%==-h (
      set optHelp=true
   ) else (

   if %argv%==--help (
      set optHelp=true
   ) else (

   if %argv%==--set-version (
      set optSetVersion=true
      echo. "%2" |findstr /r "[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*" >nul
      if errorlevel 1 (
         call :ErrMessage "invalid argument for option %argv%, got '%2'; see --help for usage."
         goto :ErrorExit
      )
rem There's no need to cache the version; if unspecified the current version will be retained by UpdateVersionNumber.py.
rem      call :SetValueInCache "version" %2
      set CameraVersion=%2
      shift
   ) else (

   if %argv%==--no-svn-update (
      set optNoSvnUpdate=true
   ) else (

   if %argv%==--version (
      set optVersion=true
   ) else (

   if %argv%==-q (
      set optQuiet=true
   ) else (

   if %argv%==--quiet (
      set optQuiet=true
   ) else (

   if %argv%==-v (
      set optVerbose=true
   ) else (

   if %argv%==--verbose (
      set optVerbose=true
   ) else (

   if %argv%==--log-level (
      set optLogLevel=true
      set logLevel=%2
      echo. "debug,info,warning,error,critical" |findstr /l "%2" >nul
      if errorlevel 1 (
         call :ErrMessage "invalid argument for option %argv%, got '%2'; see --help for usage."
         goto :ErrorExit
      )
      shift
   ) else (
      call :ErrMessage "unrecognised option '%argv%'; see --help for usage."
      goto :ErrorExit
   ) ) ) ) ) ) ) ) ) )
   shift
   goto :OptLoop
:EndOptLoop

::
:: Summarise, perform actions for collected options
::
:Summarise
if defined optHelp (
   call :Usage %progname% & exit /b 0
)
if defined optVersion (
   call :Version %progname% & ( cd Project & call CheckVsProjects.bat --version ) &  exit /b 0
)

rem Cache is currently not used; enable following line if it is:
rem call :SetValueInCache "description" "This file contains cached values from the BatchSkeleton.bat script."

rem There's no need to cache the version; if unspecified the current version will be retained by UpdateVersionNumber.py.
rem call :GetCachedCameraVersion CameraVersion

if [%logLevel%]==[debug] (
   call :ErrMessage "optSetVersion: '%optSetVersion%'"
   call :ErrMessage "CameraVersion: '%CameraVersion%'"

   call :ErrMessage "optNoSvnUpdate: '%optNoSvnUpdate%'"

   call :ErrMessage "optQuiet: '%optQuiet%'"
   call :ErrMessage "optVerbose: '%optVerbose%'"

   call :ErrMessage "version: %version%
   call :ErrMessage "versionDate: %versionDate%

   call :ErrMessage "progname: %progname%
   call :ErrMessage "logname: %logname%
   call :ErrMessage "logpath: %logpath%
)

::
:: C. Process Files
::
echo BatchSkeleton %version% (%versionDate%)
echo.

set          BeforeCmd=call :Before
set ProcessAllFilesCmd=call :ProcessAllFiles
set     ProcessFileCmd=call :ProcessFile
set           AfterCmd=call :After
set             LogCmd=call :LogMessage

set /a fileCount=0

:: Note: To be able to shift all (posibly more than 9 arguments), need access
:: to original arguments below
:: Chain commands to propagate error level.
::
%BeforeCmd%        &&^
:ProcessAllFiles   &&^
if not defined nil (
   :FileLoop
   set argv=%1& rem no trailing space!
   if [%argv%]==[] goto :EndFileLoop
   for %%f in ( %argv% ) do (
      %ProcessFileCmd% %%f
      set /a fileCount+=1
   )
   shift
   goto :FileLoop
)  &&^
:EndFileLoop  &&^
%AfterCmd%    &&^
%LogCmd% "Processed %filecount% file(s)"

endlocal & goto :EOF

::
:: Step X: ProcessFile( path )
::
:ProcessFile
setlocal
set filename=%1& rem no trailing space
call :Basename %filename% basename
%LogCmd% "Processing %filename%:"
type x
echo EL:%errorlevel%
endlocal & goto :EOF

::-------------------------------------
:: D. Subroutines
::

::
:: Step 0: Before()
::
:Before
setlocal
call :Now today now
echo [%today% %now%] - %logname%: created on %today% %now%  2>&1 > %logpath%

call :CompilerVersion version
if not [%version%]==[1200] (
   call :ErrMessage "Expected Microsoft compiler version 1200 (Visual Studio 6), got %version%."
   call :ErrMessage "Please setup Visual Studio 6 environment first."
   goto ErrorExit
)
endlocal & goto :EOF

::
:: Step 77: After()
::
:After
setlocal
call :Now today now
echo.                                                        2>&1 >> %logpath%
echo [%today% %now%] - %logname%: finished on %today% %now%  2>&1 >> %logpath%
call :LogMessage "Done."
endlocal & goto :EOF

::
:: Step 1: SvnUdate( folder, doUpdate... )
::
:SvnUpdate
setlocal
call :LogMessage "1. Updating to latest Subversion revision..."
set folder="%1" & shift
( cd "%folder%" & %1 %2 %3 %4 %5 )  2>&1 >> "%logpath%"
if errorlevel 1 (
   call :ErrMessage "error while updating to latest Subversion revision; see log file '%logname%'."
   goto :ErrorExit
)
endlocal & goto :EOF

:NoSvnUpdate
setlocal
call :LogMessage "1. Updating to latest Subversion revision...skipped"
endlocal & goto :EOF

::
:: Subroutine Version( program_name )
:: Show program's version.
::
:Version program_name
setlocal
echo %progname%  %version% (%versionDate%)
endlocal & goto :EOF

::
:: Subroutine Usage( program_name )
:: Show program's usage.
::
:Usage program_name
setlocal
set progname=%1
echo.
echo Usage: %progname% [options]
echo.
echo Options
echo.
echo --help, -h
echo        this help page
echo.
echo --set-version={version}
echo        specify new version as 1.2.3. If left unspecified, the current version is retained.
echo.
echo --no-svn-update
echo        do not update the working copy to the latest subversion revision; reduces compilation time.
echo.
echo --version
echo        report the versions of this script and other scripts used.
echo.
echo --quiet, -q
echo        suppress reporting filenames and number of files processed.
echo.
echo --verbose, -v
echo        also report paths of intermediate files and number of files processed.
echo.
echo --log-level={level}
echo        specify as one of debug, info, warning, error, critical [info].
echo.
echo %progname% performs the following [describe]
endlocal & goto :EOF

::
:: Subroutine CompilerVersion( [out]version )
::
:: Determine Microsoft Compiler version.
::
:CompilerVersion version
@echo off & setlocal enableextensions
set tmpprogram=_getcompilerversion.tmp
set tmpsource=%tmpprogram%.c

echo #include ^<stdio.h^>                   >%tmpsource%
echo int main(){printf("%%d\n",_MSC_VER);} >>%tmpsource%

cl /nologo %tmpsource% >nul
for /f %%x in ('%tmpprogram%') do set version=%%x
del %tmpprogram%.* >nul
endlocal & set %1=%version%& goto :EOF

::
:: Subroutine GetCachedCameraVersion( [out]version )
::
:GetCachedCameraVersion
setlocal enableextensions
call :GetCachedValue "version" result
endlocal & set %1=%result%& goto :EOF

::
:: Subroutine SetValueInCache( key, value )
::
:SetValueInCache
setlocal enableextensions
call :Unquote %1 key
call :Unquote %2 value
echo %key%:%value% > "%cachepath%.tmp"
findstr /l /b /v %key% "%cachepath%" >> "%cachepath%.tmp"
move /y "%cachepath%.tmp" "%cachepath%"
endlocal & goto :EOF

::
:: Subroutine GetCachedValue( [in]key, [out]value )
::
:GetCachedValue
setlocal enableextensions
call :Unquote %1 key
set result=None
if exist %cachepath% (
   for /f "tokens=1,2* delims=:" %%f in ('findstr /l /b %key% %cachepath%') do (
      set result=%%g
   )
)
endlocal & set %2=%result%& goto :EOF

::
:: Subroutine Unquote( [in]str, [out]str )
::
:Unquote
setlocal enableextensions
set string=%1
set result=%string:"=%
endlocal & set %2=%result%& goto :EOF

::
:: Subroutine Basename( [in]path, [out]basename )
::
:Basename
setlocal enableextensions
for %%f in (%1) do set basename=%%~nxf
endlocal & set %2=%basename%& goto :EOF

::
:: Subroutine FullPath( [in]path, [out]fullpath )
::
:FullPath
setlocal enableextensions
for %%f in (%1) do set fullpath=%%~ff
endlocal & set %2=%fullpath%& goto :EOF

::
:: Subroutine Now( [out]date, [out]time )
::
:Now
setlocal enableextensions
endlocal & set %1=%DATE%& set %2=%TIME:~0,-3%& goto :EOF

::
:: Subroutine ErrMessage( [in]message )
::
:: Strip quotes from message, issue it
::
:ErrMessage
setlocal
set message=%1
set message=%message:"=%
echo %progName%: %message%
endlocal & goto :EOF

::
:: Subroutine LogMessage( [in]message )
::
:: Strip quotes from message, issue and log it
::
:LogMessage
setlocal
call :Now today now
call :Unquote %1 message
if not defined optQuiet echo %now% - %message%
echo.                                          2>&1 >> "%logpath%"
echo.                                          2>&1 >> "%logpath%"
echo [%today% %now%] - %progName%: %message%   2>&1 >> "%logpath%"
echo.                                          2>&1 >> "%logpath%"
endlocal & goto :EOF

::
:: Exit with errorlevel 1 for errors.
::
:ErrorExit
endlocal
exit /b 1

::
:: end of file
::
